# Copyright (C) 1995 John W. Eaton
# 
# This file is part of Octave.
# 
# Octave is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# Octave is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Octave; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

function saveimg (filename, X, img_form, map)

# Save a matrix to disk in image format.
# 
# saveimg (filename, x) saves matrix x to file filename in octave's
# image format.  The current colormap is saved in the file also.
# 
# saveimg (filename, x, "img") saves the image in the default format
# and is the same as saveimg (filename, x).
# 
# saveimg (filename, x, "ppm") saves the image in ppm format instead
# of the default octave image format.
# 
# saveimg (filename, x, "ps") saves the image in PostScript format
# instead of the default octave image format. (Note: images saved in
# PostScript format can not be read back into octave with loadimg.)
# 
# saveimg (filename, x, format, map) saves the image along with the
# specified colormap in the specified format.
# 
# Note: If the colormap contains only two entries and these entries
# are black and white, the bitmap ppm and PostScript formats are used.
# If the image is a gray scale image (the entries within each row of
# the colormap are equal) the gray scale ppm and PostScript image
# formats are used, otherwise the full color formats are used.
# 
# SEE ALSO: loadimg, save, load, colormap

# Written by Tony Richardson (amr@mpl.ucsd.edu) July 1994.
# Modified by Klaus Gebhardt, 1995

  if (nargin < 2 || nargin > 4)
    usage ("saveimg (filename, matrix, [format, [colormap]])");
  endif

  if (! isstr (filename))
    error ("file name must be a string");
  endif
  
  if (nargin == 2)
    map = colormap;
    img_form = "img";
  elseif (nargin == 3)
    if (! isstr (img_form))
      error ("image format specification must be a string");
    endif
    map = colormap;
  endif

# XXX FIXME XXX -- we should check the remaining args.

  if (strcmp (img_form, "img") == 1)
    oct_file = filename;
  else
    oct_file = octave_tmp_file_name ();
  endif

# Save image in octave image file format

  eval (['save ', oct_file, ' map X']);

# Convert to another format if requested.

  if (strcmp (img_form, "img") == 0)
    rm = sprintf("rm -f %s", oct_file);
    if (strcmp (img_form, "ppm") == 1)
      conv_cmd = sprintf ("octtopnm %s > %s", oct_file, filename);
    elseif (strcmp (img_form, "ps") == 1)
      conv_cmd = sprintf ("octtopnm %s | pnmtops >%s", oct_file, filename);
    elseif (strcmp (img_form, "gif") == 1)
      conv_cmd = sprintf ("octtopnm %s | pnmtogif >%s", oct_file, filename);
    endif
    shell_cmd (conv_cmd);
    shell_cmd (rm);
  endif
  
endfunction
